using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// EmbersEditor v1.0
//
// Allows you to edit some parameters of Embers particles

namespace EmbersEditorPlugin
{
    public class EmbersEditor : MVRScript
    {
		public JSONStorableColor EmbersMainColor1;
		public JSONStorableColor EmbersMainColor2;
		public JSONStorableFloat EmbersMainOpacity1;
		public JSONStorableFloat EmbersMainOpacity2;
		public JSONStorableFloat EmbersEmissionRate;
		public JSONStorableFloat EmbersMainMaxParticles;
		public JSONStorableFloat EmbersScaleRadius;
		public JSONStorableFloat EmbersScaleDonutRadius;
		public JSONStorableFloat EmbersMainLifetimeMin;
		public JSONStorableFloat EmbersMainLifetimeMax;
		public JSONStorableFloat EmbersMainSimulationSpeed;
		
		public JSONStorableFloat ParticlesSortingPriority;
		
		public Transform ParticleRoot;
		private ParticleSystem prtComponent;
		private ParticleSystemRenderer prtRendererComponent;
		private string objectSearchName = "_editable";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add EmbersEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }
				
				HSVColor hsvc1 = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				EmbersMainColor1 = new JSONStorableColor("Embers color 1", hsvc1, MainColor1Callback);
				RegisterColor(EmbersMainColor1);
				CreateColorPicker(EmbersMainColor1);
							
				EmbersMainOpacity1 = new JSONStorableFloat("Embers opacity 1", 1f, MainOpacity1Callback, 0f, 1f);
				RegisterFloat(EmbersMainOpacity1);
				CreateSlider(EmbersMainOpacity1, false);

				HSVColor hsvc2 = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				EmbersMainColor2 = new JSONStorableColor("Embers color 2", hsvc2, MainColor2Callback);
				RegisterColor(EmbersMainColor2);
				CreateColorPicker(EmbersMainColor2);

				EmbersMainOpacity2 = new JSONStorableFloat("Embers opacity 2", 1f, MainOpacity2Callback, 0f, 1f);
				RegisterFloat(EmbersMainOpacity2);
				CreateSlider(EmbersMainOpacity2, false);
				
				CreateSpacer();
				
				EmbersEmissionRate = new JSONStorableFloat("Embers emission rate", 100f, EmissionRateCallback, 1f, 5000f);
				RegisterFloat(EmbersEmissionRate);
				CreateSlider(EmbersEmissionRate, false);
				
				EmbersMainMaxParticles = new JSONStorableFloat("Max Embers", 10000f, MainMaxParticlesCallback, 1f, 40000f);
				RegisterFloat(EmbersMainMaxParticles);
				CreateSlider(EmbersMainMaxParticles, false);
				
				EmbersScaleRadius = new JSONStorableFloat("Embers tube radius", 1, ShapeScaleRadiusCallback, 0.01f, 50f);
				RegisterFloat(EmbersScaleRadius);
				CreateSlider(EmbersScaleRadius, false);
				
				EmbersScaleDonutRadius = new JSONStorableFloat("Embers tube thickness", 0.2f, ShapeScaleDonutRadiusCallback, 0.05f, 5f);
				RegisterFloat(EmbersScaleDonutRadius);
				CreateSlider(EmbersScaleDonutRadius, false);
				
				EmbersMainLifetimeMin = new JSONStorableFloat("Min Lifetime", 2, MainLifetimeMinCallback, 0.1f, 10);
				RegisterFloat(EmbersMainLifetimeMin);
				CreateSlider(EmbersMainLifetimeMin, false);
				
				EmbersMainLifetimeMax = new JSONStorableFloat("Max Lifetime", 2, MainLifetimeMaxCallback, 0.1f, 10);
				RegisterFloat(EmbersMainLifetimeMax);
				CreateSlider(EmbersMainLifetimeMax, false);
		
				
				EmbersMainSimulationSpeed = new JSONStorableFloat("Simulation speed", 1, MainSimulationSpeedCallback, 0.01f, 20);
				RegisterFloat(EmbersMainSimulationSpeed);
				CreateSlider(EmbersMainSimulationSpeed, false);
				
				ParticlesSortingPriority = new JSONStorableFloat("Particles priority", 0, SortingPriorityCallback, -1f, 1f);
				RegisterFloat(ParticlesSortingPriority);
				CreateSlider(ParticlesSortingPriority, false);
							
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>EmbersEditor</b> is a tiny plugin that allows you to edit some parameters of the Embers particles inside this package\n\n" +
					"Just test the parameters on the left, and the particle will update in realtime.\n\n" +
					"With multiple particle systems, if you want some particles to be in front of the others use <b>Particles priority.</b> Lower values are displayed in the front, higher values are displayed in the back.\n\n" +
					"<b>IMPORTANT :</b> If you switch particles in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 850.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("EmbersEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitDE();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("EmbersEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitDE() {
			if( seInitialized == true ) return true;
			
			ParticleRoot = getObjectRoot( containingAtom.reParentObject );
			if( ParticleRoot == null ) {
				return false;
			}
			
			prtComponent = ParticleRoot.GetComponent<ParticleSystem>();
			if( prtComponent ) {
				prtRendererComponent = ParticleRoot.GetComponent<ParticleSystemRenderer>();
				seInitialized = true;
				
				MainColor1Callback( EmbersMainColor1 );

				EmissionRateCallback( EmbersEmissionRate.val );
				MainMaxParticlesCallback( EmbersMainMaxParticles.val );
				
				ShapeScaleRadiusCallback( EmbersScaleRadius.val );
				ShapeScaleDonutRadiusCallback( EmbersScaleDonutRadius.val );
				
				MainLifetimeMinCallback( EmbersMainLifetimeMin.val );
							
				MainSimulationSpeedCallback( EmbersMainSimulationSpeed.val );
				
				SortingPriorityCallback( ParticlesSortingPriority.val );
				
				logDebug("EmbersEditor : Embers editor initialized properly");
							
				return true;
			} else {
				logError("EmbersEditor : your CUA has a transform with _editable but does not have a ParticleSystem component attached.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectSearchName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void MainColor1Callback(JSONStorableColor color) {
			if( prtComponent ) {
				Color startColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, EmbersMainOpacity1.val);
				Color endColor = new Color(EmbersMainColor2.colorPicker.currentColor.r, EmbersMainColor2.colorPicker.currentColor.g, EmbersMainColor2.colorPicker.currentColor.b, EmbersMainOpacity2.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}
		
		protected void MainOpacity1Callback(float opacity) {
			if( prtComponent ) {
				Color startColor = new Color(EmbersMainColor1.colorPicker.currentColor.r, EmbersMainColor1.colorPicker.currentColor.g, EmbersMainColor1.colorPicker.currentColor.b, opacity);
				Color endColor = new Color(EmbersMainColor2.colorPicker.currentColor.r, EmbersMainColor2.colorPicker.currentColor.g, EmbersMainColor2.colorPicker.currentColor.b, EmbersMainOpacity2.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}

		protected void MainColor2Callback(JSONStorableColor color) {
			if( prtComponent ) {
				Color startColor = new Color(EmbersMainColor1.colorPicker.currentColor.r, EmbersMainColor1.colorPicker.currentColor.g, EmbersMainColor1.colorPicker.currentColor.b, EmbersMainOpacity1.val);
				Color endColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, EmbersMainOpacity2.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}
		
		protected void MainOpacity2Callback(float opacity) {
			if( prtComponent ) {
				Color startColor = new Color(EmbersMainColor1.colorPicker.currentColor.r, EmbersMainColor1.colorPicker.currentColor.g, EmbersMainColor1.colorPicker.currentColor.b, EmbersMainOpacity1.val);
				Color endColor = new Color(EmbersMainColor2.colorPicker.currentColor.r, EmbersMainColor2.colorPicker.currentColor.g, EmbersMainColor2.colorPicker.currentColor.b, opacity);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}
		
		protected void EmissionRateCallback(float rate) {
			if( prtComponent ) {
				var prtEmission = prtComponent.emission;
				prtEmission.rate = rate;
			}
		}
		
		protected void MainMaxParticlesCallback(float maxprts) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.maxParticles = (int)maxprts;
			}
		}
		
		protected void ShapeScaleRadiusCallback(float scale) {
			if( prtComponent ) {
				var prtShape = prtComponent.shape;
				prtShape.radius = scale;
				
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void ShapeScaleDonutRadiusCallback(float scale) {
			if( prtComponent ) {
				var prtShape = prtComponent.shape;
				prtShape.donutRadius = scale;
				
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void MainLifetimeMinCallback(float minlifetime) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startLifetime = new ParticleSystem.MinMaxCurve(minlifetime, EmbersMainLifetimeMax.val);
			}
		}
		
		protected void MainLifetimeMaxCallback(float maxlifetime) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startLifetime = new ParticleSystem.MinMaxCurve(EmbersMainLifetimeMin.val, maxlifetime);
			}
		}
		
		protected void MainSimulationSpeedCallback(float simspeed) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.simulationSpeed = simspeed;
			}
		}
		
		protected void SortingPriorityCallback(float priority) {
			if( prtRendererComponent ) {
				prtRendererComponent.sortingFudge = priority;
			}
		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}