using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// ForceField01Editor v1.0
//
// Allows you to edit some parameters of Force Field 01

namespace ForceField01EditorPlugin
{
    public class ForceField01Editor : MVRScript
    {
		public JSONStorableColor ForceFieldColor;
		public JSONStorableFloat ForceFieldColorIntensity;
		public JSONStorableFloat ForceFieldColorShape;
		
		public JSONStorableFloat ForceFieldDistortionScale;
		public JSONStorableFloat ForceFieldRefractionIntensity;
		
		public JSONStorableFloat ForceFieldRefractionSizeX;
		public JSONStorableFloat ForceFieldRefractionSizeY;
		
		public Transform ObjectRoot;
		private Renderer objRenderer;
		private string objectSearchName = "_editable";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add ForceField01Editor on an Custom Unity Asset with a compatible object.");
                    return;
                }
				
				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				ForceFieldColor = new JSONStorableColor("ForceField color", hsvc, ForceFieldColorCallback);
				RegisterColor(ForceFieldColor);
				CreateColorPicker(ForceFieldColor);
				
				ForceFieldColorIntensity = new JSONStorableFloat("ForceField color intensity", 10.0f, ForceFieldColorIntensityCallback, 0f, 50f);
				RegisterFloat(ForceFieldColorIntensity);
				CreateSlider(ForceFieldColorIntensity, false);
				
				ForceFieldColorShape = new JSONStorableFloat("ForceField color shape", 5.0f, ForceFieldColorShapeCallback, 1f, 50f);
				RegisterFloat(ForceFieldColorShape);
				CreateSlider(ForceFieldColorShape, false);
				
				ForceFieldDistortionScale = new JSONStorableFloat("ForceField distortion scale", 0.1f, ForceFieldDistortionScaleCallback, 0.0f, 1.0f);
				RegisterFloat(ForceFieldDistortionScale);
				CreateSlider(ForceFieldDistortionScale, false);
				
				ForceFieldRefractionIntensity = new JSONStorableFloat("ForceField refraction intensity", 0.2f, ForceFieldRefractionIntensityCallback, 0f, 1.0f);
				RegisterFloat(ForceFieldRefractionIntensity);
				CreateSlider(ForceFieldRefractionIntensity, false);

				ForceFieldRefractionSizeX = new JSONStorableFloat("ForceField refraction size X", 0.5f, ForceFieldRefractionSizeXCallback, 0.1f, 50.0f);
				RegisterFloat(ForceFieldRefractionSizeX);
				CreateSlider(ForceFieldRefractionSizeX, false);
				
				ForceFieldRefractionSizeY = new JSONStorableFloat("ForceField refraction size Y", 0.5f, ForceFieldRefractionSizeXCallback, 0.1f, 50.0f);
				RegisterFloat(ForceFieldRefractionSizeY);
				CreateSlider(ForceFieldRefractionSizeY, false);			
										
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>ForceField01Editor</b> is a tiny plugin that allows you to edit some parameters of the Force Field 01 object inside this package\n\n" +
					"Just test the parameters on the left, and the object will update in realtime.\n\n" +
					"<b>IMPORTANT :</b> If you switch objects in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("ForceField01Editor - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitEditor();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("ForceField01Editor : This CUA could not be initialized properly after 3600 tries. Please select a compatible object and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitEditor() {
			if( seInitialized == true ) return true;
			
			ObjectRoot = getObjectRoot( containingAtom.reParentObject );
			if( ObjectRoot == null ) {
				return false;
			}
			
			objRenderer = ObjectRoot.GetComponent<Renderer>();
			if( objRenderer ) {
				seInitialized = true;
				
				ForceFieldColorCallback( ForceFieldColor );
				ForceFieldColorIntensityCallback(ForceFieldColorIntensity);
				ForceFieldColorShapeCallback(ForceFieldColorShape);
				ForceFieldDistortionScaleCallback(ForceFieldDistortionScale);
				ForceFieldRefractionIntensityCallback(ForceFieldRefractionIntensity);
				ForceFieldRefractionSizeXCallback(ForceFieldRefractionSizeX);
				ForceFieldRefractionSizeYCallback(ForceFieldRefractionSizeY);
							
				logDebug("ForceField01Editor : Force Field editor initialized properly");
							
				return true;
			} else {
				logError("ForceField01Editor : your CUA has a transform with _editable but does not have a ParticleSystem component attached.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectSearchName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void ForceFieldColorCallback(JSONStorableColor color) {
			if( objRenderer ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, 1.0f);
				objRenderer.material.SetColor("_FieldColor",newColor);
			}
		}
		
		protected void ForceFieldColorIntensityCallback(JSONStorableFloat opacity) {
			if( objRenderer ) {
				objRenderer.material.SetFloat("_FieldColorIntensity",opacity.val);
			}
		}
		
		protected void ForceFieldColorShapeCallback(JSONStorableFloat shape) {
			if( objRenderer ) {
				objRenderer.material.SetFloat("_FieldShape",shape.val);
			}
		}
		
		protected void ForceFieldDistortionScaleCallback(JSONStorableFloat scale) {
			if( objRenderer ) {
				objRenderer.material.SetFloat("_FieldDistortionScale",scale.val);
			}
		}
		
		protected void ForceFieldRefractionIntensityCallback(JSONStorableFloat intensity) {
			if( objRenderer ) {
				objRenderer.material.SetFloat("_FieldRefractionIntensity",intensity.val);
			}
		}
		
		protected void ForceFieldRefractionSizeXCallback(JSONStorableFloat sizeX) {
			if( objRenderer ) {
				objRenderer.material.SetTextureScale("_Normals", new Vector2(sizeX.val, ForceFieldRefractionSizeY.val));
			}
		}
		
		protected void ForceFieldRefractionSizeYCallback(JSONStorableFloat sizeY) {
			if( objRenderer ) {
				objRenderer.material.SetTextureScale("_Normals", new Vector2(ForceFieldRefractionSizeX.val, sizeY.val));
			}
		}
		

		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}