using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// LightningEditor v1.0
//
// Allows you to edit some parameters of lightning particles

namespace LightningEditorPlugin
{
    public class LightningEditor : MVRScript
    {
		public JSONStorableColor LightningMainColor;
		public JSONStorableFloat LightningMainOpacity;
		public JSONStorableColor LightningStartColor;
		public JSONStorableColor LightningEndColor;
		public JSONStorableFloat LightningMainMaxParticles;
		public JSONStorableFloat LightningMainLifetimeMin;
		public JSONStorableFloat LightningMainLifetimeMax;
		public JSONStorableFloat LightningMainSizeMin;
		public JSONStorableFloat LightningMainSizeMax;
		public JSONStorableFloat LightningMainSimulationSpeed;
		public JSONStorableFloat LightningVelocitySpeedModifier;
		
		public JSONStorableFloat ParticlesSortingPriority;
		
		public Transform ParticleRoot;
		private ParticleSystem prtComponent;
		private ParticleSystemRenderer prtRendererComponent;
		private TrailRenderer trailRendererComponent;
		private string objectSearchName = "_editable";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add LightningEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }
				
				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				LightningMainColor = new JSONStorableColor("Lightning color", hsvc, MainColorCallback);
				RegisterColor(LightningMainColor);
				CreateColorPicker(LightningMainColor);
				
				LightningMainOpacity = new JSONStorableFloat("Lightning opacity", 1.0f, MainOpacityCallback, 0f, 1f);
				RegisterFloat(LightningMainOpacity);
				CreateSlider(LightningMainOpacity, false);
				
				HSVColor hsvcstart = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				LightningStartColor = new JSONStorableColor("Lightning start color", hsvcstart, StartColorCallback);
				RegisterColor(LightningStartColor);
				CreateColorPicker(LightningStartColor);
				
				HSVColor hsvcend = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				LightningEndColor = new JSONStorableColor("Lightning end color", hsvcend, EndColorCallback);
				RegisterColor(LightningEndColor);
				CreateColorPicker(LightningEndColor);

							
				LightningMainMaxParticles = new JSONStorableFloat("Max Lightning", 5000, MainMaxParticlesCallback, 1f, 5000f);
				RegisterFloat(LightningMainMaxParticles);
				CreateSlider(LightningMainMaxParticles, false);
				
				LightningMainLifetimeMin = new JSONStorableFloat("Min Lifetime", 0.2f, MainLifetimeMinCallback, 0.05f, 5f);
				RegisterFloat(LightningMainLifetimeMin);
				CreateSlider(LightningMainLifetimeMin, false);
				
				LightningMainLifetimeMax = new JSONStorableFloat("Max Lifetime", 0.3f, MainLifetimeMaxCallback, 0.05f, 5f);
				RegisterFloat(LightningMainLifetimeMax);
				CreateSlider(LightningMainLifetimeMax, false);
				
				LightningMainSizeMin = new JSONStorableFloat("Min Size", 1, MainSizeMinCallback, 1, 20);
				RegisterFloat(LightningMainSizeMin);
				CreateSlider(LightningMainSizeMin, false);
				
				LightningMainSizeMax = new JSONStorableFloat("Max Size", 10, MainSizeMaxCallback, 1, 20);
				RegisterFloat(LightningMainSizeMax);
				CreateSlider(LightningMainSizeMax, false);				
				
				LightningMainSimulationSpeed = new JSONStorableFloat("Simulation speed", 1, MainSimulationSpeedCallback, 0.01f, 20);
				RegisterFloat(LightningMainSimulationSpeed);
				CreateSlider(LightningMainSimulationSpeed, false);
				
				LightningVelocitySpeedModifier = new JSONStorableFloat("Lightning speed modifier", 1, VelocitySpeedModifierCallback, 0.01f, 10, true);
				RegisterFloat(LightningVelocitySpeedModifier);
				CreateSlider(LightningVelocitySpeedModifier, false);
							
				ParticlesSortingPriority = new JSONStorableFloat("Particles priority", 0, SortingPriorityCallback, -1f, 1f);
				RegisterFloat(ParticlesSortingPriority);
				CreateSlider(ParticlesSortingPriority, false);
				
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>LightningEditor</b> is a tiny plugin that allows you to edit some parameters of the Lightning particles inside this package.\n\n" +
					"Just test the parameters on the left, and the particle will update in realtime.\n\n" +
					"With multiple particle systems, if you want some particles to be in front of the others use <b>Particles priority.</b> Lower values are displayed in the front, higher values are displayed in the back.\n\n" +
					"<b>For creators and screenshot/still artists :</b> Be careful. I cannot explain why, but the vanilla screenshot feature of VAM does not render exactly the same as what you see on the realtime viewport. " +
					"In my case, I used virtual resolution to upscale the game and make higher res screenshots.\n\n" +
					"<b>IMPORTANT :</b> If you switch particles in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("LightningEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitDE();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("LightningEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitDE() {
			if( seInitialized == true ) return true;
			
			ParticleRoot = getObjectRoot( containingAtom.reParentObject );
			if( ParticleRoot == null ) {
				return false;
			}
			
			prtComponent = ParticleRoot.GetComponent<ParticleSystem>();
			if( prtComponent ) {		
				prtRendererComponent = ParticleRoot.GetComponent<ParticleSystemRenderer>();
				trailRendererComponent = ParticleRoot.GetComponent<TrailRenderer>();
				seInitialized = true;
				
				MainColorCallback( LightningMainColor );
				MainOpacityCallback( LightningMainOpacity.val );
				StartColorCallback( LightningStartColor );
				EndColorCallback( LightningEndColor );
				MainMaxParticlesCallback( LightningMainMaxParticles.val );
				
				MainLifetimeMinCallback( LightningMainLifetimeMin.val );
				
				MainSizeMinCallback( LightningMainSizeMin.val );
				
				MainSimulationSpeedCallback( LightningMainSimulationSpeed.val );
				VelocitySpeedModifierCallback( LightningVelocitySpeedModifier.val );
				
				SortingPriorityCallback( ParticlesSortingPriority.val );
				
				logDebug("LightningEditor : Lightning editor initialized properly");
							
				return true;
			} else {
				logError("LightningEditor : your CUA has a transform with _editable but does not have a ParticleSystem component attached.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectSearchName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void MainColorCallback(JSONStorableColor color) {
			if( prtComponent ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, LightningMainOpacity.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = newColor;
			}
		}
		
		protected void MainOpacityCallback(float opacity) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startColor = new Color(LightningMainColor.colorPicker.currentColor.r, LightningMainColor.colorPicker.currentColor.g, LightningMainColor.colorPicker.currentColor.b, opacity);
			}
		}
		
		protected void StartColorCallback(JSONStorableColor color) {
			if( prtComponent ) {
				var prtLightning = prtComponent.trails;
				
				Color startColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, 1f );
				Color endColor = new Color(LightningEndColor.colorPicker.currentColor.r, LightningEndColor.colorPicker.currentColor.g, LightningEndColor.colorPicker.currentColor.b, 0f);
				
				Gradient gradient = new Gradient();
				gradient.SetKeys(
					new GradientColorKey[] { new GradientColorKey(startColor, 0.0f), new GradientColorKey(endColor, 1.0f) },
					new GradientAlphaKey[] { new GradientAlphaKey(1f, 0f), new GradientAlphaKey(0f, 1.0f) }
				);
				prtLightning.colorOverTrail = gradient;
			}
		}
		
		protected void EndColorCallback(JSONStorableColor color) {
			if( prtComponent ) {
				var prtLightning = prtComponent.trails;
				
				Color startColor = new Color(LightningStartColor.colorPicker.currentColor.r, LightningStartColor.colorPicker.currentColor.g, LightningStartColor.colorPicker.currentColor.b, 1f );
				Color endColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, 0f );
				
				Gradient gradient = new Gradient();
				gradient.SetKeys(
					new GradientColorKey[] { new GradientColorKey(startColor, 0.0f), new GradientColorKey(endColor, 1.0f) },
					new GradientAlphaKey[] { new GradientAlphaKey(1f, 0f), new GradientAlphaKey(0f, 1.0f) }
				);
				prtLightning.colorOverTrail = gradient;
			}
		}
			
		protected void MainMaxParticlesCallback(float maxprts) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.maxParticles = (int)maxprts;
			}
		}
		
		protected void MainLifetimeMinCallback(float minlifetime) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startLifetime = new ParticleSystem.MinMaxCurve(minlifetime, LightningMainLifetimeMax.val);
			}
		}
		
		protected void MainLifetimeMaxCallback(float maxlifetime) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startLifetime = new ParticleSystem.MinMaxCurve(LightningMainLifetimeMin.val, maxlifetime);
			}
		}
		
		protected void MainSizeMinCallback(float minsize) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startSize = new ParticleSystem.MinMaxCurve(minsize, LightningMainSizeMax.val);
			}
		}
		
		protected void MainSizeMaxCallback(float maxsize) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startSize = new ParticleSystem.MinMaxCurve(LightningMainSizeMin.val, maxsize);
			}
		}
		
		protected void MainSimulationSpeedCallback(float simspeed) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.simulationSpeed = simspeed;
			}
		}
		
		protected void VelocitySpeedModifierCallback(float speedmult) {
			if( prtComponent ) {
				var prtVelocity = prtComponent.velocityOverLifetime;
				prtVelocity.speedModifier = speedmult;
			}
		}
		
		protected void SortingPriorityCallback(float priority) {
			if( prtRendererComponent ) {
				prtRendererComponent.sortingFudge = priority;
			}
		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}