using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// SmokeEditor v1.0
//
// Allows you to edit some parameters of smoke particles

namespace SmokeEditorPlugin
{
    public class SmokeEditor : MVRScript
    {
		public JSONStorableColor SmokeMainColor1;
		public JSONStorableFloat SmokeMainOpacity1;
		public JSONStorableColor SmokeMainColor2;
		public JSONStorableFloat SmokeMainOpacity2;
		public JSONStorableFloat SmokeEmissionRate;
		public JSONStorableFloat SmokeMainMaxParticles;
		public JSONStorableFloat SmokeMainLifetimeMin;
		public JSONStorableFloat SmokeMainLifetimeMax;
		public JSONStorableFloat SmokeMainSizeMin;
		public JSONStorableFloat SmokeMainSizeMax;
		public JSONStorableFloat SmokeRotationMin;
		public JSONStorableFloat SmokeRotationMax;
		public JSONStorableFloat SmokeMainSimulationSpeed;
		
		public JSONStorableFloat ParticlesSortingPriority;
		
		public Transform ParticleRoot;
		private ParticleSystem prtComponent;
		private ParticleSystemRenderer prtRendererComponent;		
		private string objectSearchName = "_editable";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add SmokeEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }
				
				HSVColor hsvc1 = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				SmokeMainColor1 = new JSONStorableColor("Smoke color 1", hsvc1, MainColor1Callback);
				RegisterColor(SmokeMainColor1);
				CreateColorPicker(SmokeMainColor1);
				
				SmokeMainOpacity1 = new JSONStorableFloat("Smoke opacity 1", 1.0f, MainOpacity1Callback, 0f, 1f);
				RegisterFloat(SmokeMainOpacity1);
				CreateSlider(SmokeMainOpacity1, false);
				
				HSVColor hsvc2 = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				SmokeMainColor2 = new JSONStorableColor("Smoke color 2", hsvc2, MainColor2Callback);
				RegisterColor(SmokeMainColor2);
				CreateColorPicker(SmokeMainColor2);
				
				SmokeMainOpacity2 = new JSONStorableFloat("Smoke opacity 2", 1.0f, MainOpacity2Callback, 0f, 1f);
				RegisterFloat(SmokeMainOpacity2);
				CreateSlider(SmokeMainOpacity2, false);
				
				CreateSpacer();

				SmokeEmissionRate = new JSONStorableFloat("Smoke emission rate", 10f, EmissionRateCallback, 1f, 1000f);
				RegisterFloat(SmokeEmissionRate);
				CreateSlider(SmokeEmissionRate, false);
				
				SmokeMainMaxParticles = new JSONStorableFloat("Max smoke", 1000f, MainMaxParticlesCallback, 1f, 1000f);
				RegisterFloat(SmokeMainMaxParticles);
				CreateSlider(SmokeMainMaxParticles, false);
				
				SmokeMainLifetimeMin = new JSONStorableFloat("Min Lifetime", 2, MainLifetimeMinCallback, 1, 20);
				RegisterFloat(SmokeMainLifetimeMin);
				CreateSlider(SmokeMainLifetimeMin, false);
				
				SmokeMainLifetimeMax = new JSONStorableFloat("Max Lifetime", 3, MainLifetimeMaxCallback, 1, 20);
				RegisterFloat(SmokeMainLifetimeMax);
				CreateSlider(SmokeMainLifetimeMax, false);
				
				SmokeMainSizeMin = new JSONStorableFloat("Min Size", 10, MainSizeMinCallback, 10, 100);
				RegisterFloat(SmokeMainSizeMin);
				CreateSlider(SmokeMainSizeMin, false);
				
				SmokeMainSizeMax = new JSONStorableFloat("Max Size", 10, MainSizeMaxCallback, 10, 100);
				RegisterFloat(SmokeMainSizeMax);
				CreateSlider(SmokeMainSizeMax, false);

				SmokeRotationMin = new JSONStorableFloat("Min rotation", 0, RotationMinCallback, -10f, 0);
				RegisterFloat(SmokeRotationMin);
				CreateSlider(SmokeRotationMin, false);
				
				SmokeRotationMax = new JSONStorableFloat("Max rotation", 0, RotationMaxCallback, 0, 10f);
				RegisterFloat(SmokeRotationMax);
				CreateSlider(SmokeRotationMax, false);				
				
				SmokeMainSimulationSpeed = new JSONStorableFloat("Simulation speed", 1, MainSimulationSpeedCallback, 0.01f, 20);
				RegisterFloat(SmokeMainSimulationSpeed);
				CreateSlider(SmokeMainSimulationSpeed, false);
				
				ParticlesSortingPriority = new JSONStorableFloat("Particles priority", 0, SortingPriorityCallback, -1f, 1f);
				RegisterFloat(ParticlesSortingPriority);
				CreateSlider(ParticlesSortingPriority, false);
							
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>SmokeEditor</b> is a tiny plugin that allows you to edit some parameters of the smoke particles inside this package.\n\n" +
					"Just test the parameters on the left, and the particle will update in realtime.\n\n" +
					"With multiple particle systems, if you want some particles to be in front of the others use <b>Particles priority.</b> Lower values are displayed in the front, higher values are displayed in the back.\n\n" +
					"<b>For creators and screenshot/still artists :</b> Be careful. I cannot explain why, but the vanilla screenshot feature of VAM does not render exactly the same as what you see on the realtime viewport. " +
					"In my case, I used virtual resolution to upscale the game and make higher res screenshots.\n\n" +
					"<b>IMPORTANT :</b> If you switch particles in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("SmokeEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitDE();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("SmokeEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitDE() {
			if( seInitialized == true ) return true;
			
			ParticleRoot = getObjectRoot( containingAtom.reParentObject );
			if( ParticleRoot == null ) {
				return false;
			}
			
			prtComponent = ParticleRoot.GetComponent<ParticleSystem>();
			if( prtComponent ) {
				prtRendererComponent = ParticleRoot.GetComponent<ParticleSystemRenderer>();				
				seInitialized = true;
				
				MainColor1Callback( SmokeMainColor1 );
				MainOpacity1Callback( SmokeMainOpacity1.val );
				MainColor2Callback( SmokeMainColor2 );
				MainOpacity2Callback( SmokeMainOpacity2.val );
				EmissionRateCallback( SmokeEmissionRate.val );
				MainMaxParticlesCallback( SmokeMainMaxParticles.val );
				
				MainLifetimeMinCallback( SmokeMainLifetimeMin.val );
				
				MainSizeMinCallback( SmokeMainSizeMin.val );
				
				RotationMinCallback( SmokeRotationMin.val );
				RotationMaxCallback( SmokeRotationMax.val );
				
				MainSimulationSpeedCallback( SmokeMainSimulationSpeed.val );
				
				SortingPriorityCallback( ParticlesSortingPriority.val );
				
				logDebug("SmokeEditor : Smoke editor initialized properly");
							
				return true;
			} else {
				logError("SmokeEditor : your CUA has a transform with _editable but does not have a ParticleSystem component attached.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectSearchName) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		protected void MainColor1Callback(JSONStorableColor color) {
			if( prtComponent ) {
				Color startColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, SmokeMainOpacity1.val);
				Color endColor = new Color(SmokeMainColor2.colorPicker.currentColor.r, SmokeMainColor2.colorPicker.currentColor.g, SmokeMainColor2.colorPicker.currentColor.b, SmokeMainOpacity2.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}
		
		protected void MainOpacity1Callback(float opacity) {
			if( prtComponent ) {
				Color startColor = new Color(SmokeMainColor1.colorPicker.currentColor.r, SmokeMainColor1.colorPicker.currentColor.g, SmokeMainColor1.colorPicker.currentColor.b, opacity);
				Color endColor = new Color(SmokeMainColor2.colorPicker.currentColor.r, SmokeMainColor2.colorPicker.currentColor.g, SmokeMainColor2.colorPicker.currentColor.b, SmokeMainOpacity2.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}
		
		protected void MainColor2Callback(JSONStorableColor color) {
			if( prtComponent ) {
				Color startColor = new Color(SmokeMainColor1.colorPicker.currentColor.r, SmokeMainColor1.colorPicker.currentColor.g, SmokeMainColor1.colorPicker.currentColor.b, SmokeMainOpacity1.val);
				Color endColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, SmokeMainOpacity2.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}
		
		protected void MainOpacity2Callback(float opacity) {
			if( prtComponent ) {
				Color startColor = new Color(SmokeMainColor1.colorPicker.currentColor.r, SmokeMainColor1.colorPicker.currentColor.g, SmokeMainColor1.colorPicker.currentColor.b, opacity);
				Color endColor = new Color(SmokeMainColor2.colorPicker.currentColor.r, SmokeMainColor2.colorPicker.currentColor.g, SmokeMainColor2.colorPicker.currentColor.b, SmokeMainOpacity2.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = new ParticleSystem.MinMaxGradient(startColor, endColor);
			}
		}

		protected void EmissionRateCallback(float rate) {
			if( prtComponent ) {
				var prtEmission = prtComponent.emission;
				prtEmission.rate = rate;
			}
		}
		
		protected void MainMaxParticlesCallback(float maxprts) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.maxParticles = (int)maxprts;
			}
		}
		
		protected void MainLifetimeMinCallback(float minlifetime) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startLifetime = new ParticleSystem.MinMaxCurve(minlifetime, SmokeMainLifetimeMax.val);
			}
		}
		
		protected void MainLifetimeMaxCallback(float maxlifetime) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startLifetime = new ParticleSystem.MinMaxCurve(SmokeMainLifetimeMin.val, maxlifetime);
			}
		}
		
		protected void MainSizeMinCallback(float minsize) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startSize = new ParticleSystem.MinMaxCurve(minsize, SmokeMainSizeMax.val);
			}
		}
		
		protected void MainSizeMaxCallback(float maxsize) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startSize = new ParticleSystem.MinMaxCurve(SmokeMainSizeMin.val, maxsize);
			}
		}
		
		protected void RotationMinCallback(float rot) {
			if( prtComponent ) {
				var prtRotation = prtComponent.rotationOverLifetime;
				prtRotation.z = new ParticleSystem.MinMaxCurve(rot, SmokeRotationMax.val);
			}
		}
		
		protected void RotationMaxCallback(float rot) {
			if( prtComponent ) {
				var prtRotation = prtComponent.rotationOverLifetime;
				prtRotation.z = new ParticleSystem.MinMaxCurve(SmokeRotationMin.val, rot);
			}
		}
		
		protected void MainSimulationSpeedCallback(float simspeed) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.simulationSpeed = simspeed;
			}
		}
		
		protected void SortingPriorityCallback(float priority) {
			if( prtRendererComponent ) {
				prtRendererComponent.sortingFudge = priority;
			}
		}
	
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}